import { Linking, ActionSheetIOS, Alert, Platform } from 'react-native';

function openDialog(urls) {
  if (Platform.OS === 'ios') {
    urls = urls.ios;
    return Promise.all(urls.map(element => Linking.canOpenURL(element[1])))
      .then((results) => {
        return urls.filter((element, index) => results[index]);
      }).then(choices => {
        // if there is no map app on the device, recommend one to download
        if (choices.length < 1) {
          return ActionSheetIOS.showActionSheetWithOptions({
            options: ['Download Gaode Maps', 'Download Baidu Maps', 'Cancel'],
            cancelButtonIndex: 2,
            title: 'Choose the map',
          }, buttonIndex => {
            if (buttonIndex === 0) {
              Linking.openURL('https://itunes.apple.com/cn/app/gao-tu-zhuan-ye-shou-ji-tu/id461703208?mt=8');
            } else if (buttonIndex === 1) {
              Linking.openURL('https://itunes.apple.com/cn/app/bai-du-tu-shou-ji-tu-lu-xian/id452186370?mt=8');
            }
          });
        }

        return ActionSheetIOS.showActionSheetWithOptions({
          options: [...(choices.map(element => element[0])), 'Cancel'],
          cancelButtonIndex: choices.length,
          title: 'Choose the map',
        }, buttonIndex => {
          if (buttonIndex < choices.length) {
            Linking.openURL(choices[buttonIndex][1]);
          }
        });
      });
  } else if (Platform.OS === 'android') {
    urls = urls.android;
    return Promise.all(urls.map(element => Linking.canOpenURL(element[1])))
      .then((results) => {
        return urls.filter((element, index) => results[index]).map(url => ({
          text: url[0],
          onPress: () => {
            Linking.openURL(url[1]);
          },
        }));
      }).then(choices => {
        // if there is no map app on the device, recommend one to download
        if (choices.length < 1) {
          return Alert.alert('Choose the map', 'You have not installed any maps', [
            { text: 'Download Gaode Maps', onPress: () => Linking.openURL('http://mobile.amap.com') },
            { text: 'Download Baidu Maps', onPress: () => Linking.openURL('http://map.baidu.com') },
            { text: 'Cancel' }
          ]);
        }

        return Alert.alert('Choose the map', 'Please choose a map to open', [...choices, { text: 'Cancel' }]);
      });
  }
}

export default {
  options: { appName: 'MapLinking' },

  setOptions(opts) {
    this.options = { ...this.options, ...opts };
  },

  /**
   * find a specific location on the maps
   *
   * @param location {lat:40, lng: 118, type: 'gcj02'}
   * @param title    
   * @param content  
   */
  markLocation(location, title, content) {
    return openDialog({
      android: [
        [
          'Open by Gaode Maps',
          `androidamap://viewMap?sourceApplication=${this.options.appName}&poiname=${title}&lat=${location.lat}&lon=${location.lng}&dev=${location.type === 'gcj02' ? '0' : '1'}`,
        ],
        [
          'Open by Baidu Maps',
          `bdapp://map/marker?location=${location.lat},${location.lng}&coord_type=${location.type === 'gcj02' ? 'gcj02' : 'wgs84'}&title=${title}&content=${content}&src=${this.options.appName}`,
        ]
      ],
      ios: [
        [
          'Open by Gaode Maps',
          `iosamap://viewMap?sourceApplication=${this.options.appName}&poiname=${title}&lat=${location.lat}&lon=${location.lng}&dev=${location.type === 'gcj02' ? '0' : '1'}`,
        ],
        [
          'Open by Baidu Maps',
          `baidumap://map/marker?location=${location.lat},${location.lng}&coord_type=${location.type === 'gcj02' ? 'gcj02' : 'wgs84'}&title=${title}&content=${content}&src=${this.options.appName}`,
        ],
        [
          'Open by Apple Maps',
          `http://maps.apple.com/?ll=${location.lat},${location.lng}&q=${title}`,
        ],
      ],
    });
  },

  /**
   * Get the route navigation from source location to destination location
   *
   * @param srcLocation  {lat:40, lng: 118, title: 'Starting point'}
   * @param destLocation {lat:40, lng: 118, type: 'gcj02', title: 'Ending point'}
   * @param mode         drive, bus, walk
   */
  planRoute(srcLocation, distLocation, mode) {
    return openDialog({
      android: [
        [
          'Get the route by Gaode Maps',
          `androidamap://route?sourceApplication=${this.options.appName}&slat=${srcLocation && srcLocation.lat}&slon=${srcLocation && srcLocation.lng}&sname=${srcLocation && srcLocation.title}&dlat=${distLocation.lat}&dlon=${distLocation.lng}&dname=${distLocation.title}&dev=${distLocation.type === 'gcj02' ? '0' : '1'}&m=0&t=${mode === 'drive' ? '2' : (mode === 'bus' ? '1' : '4')}`,
        ],
        [
          'Get the route by Baidu Maps',
          `bdapp://map/direction?origin=${srcLocation ? (srcLocation.lat + ',' + srcLocation.lng) : ''}&destination=${distLocation.lat},${distLocation.lng}&mode=${mode === 'drive' ? 'driving' : (mode === 'bus' ? 'transit' : 'walking')}&coord_type=${distLocation.type === 'gcj02' ? 'gcj02' : 'wgs84'}&src=${this.options.appName}`,
        ]
      ],
      ios: [
        [
          'Get the route by Gaode Maps',
          `iosamap://path?sourceApplication=${this.options.appName}&slat=${srcLocation && srcLocation.lat}&slon=${srcLocation && srcLocation.lng}&sname=${srcLocation && srcLocation.title}&dlat=${distLocation.lat}&dlon=${distLocation.lng}&dname=${distLocation.title}&dev=${distLocation.type === 'gcj02' ? '0' : '1'}&t=${mode === 'drive' ? '0' : (mode === 'bus' ? '1' : '2')}`,
        ],
        [
          'Get the route by Baidu Maps',
          `baidumap://map/direction?origin=${srcLocation ? (srcLocation.lat + ',' + srcLocation.lng) : ''}&destination=${distLocation.lat},${distLocation.lng}&mode=${mode === 'drive' ? 'driving' : (mode === 'bus' ? 'transit' : 'walking')}&coord_type=${distLocation.type === 'gcj02' ? 'gcj02' : 'wgs84'}&src=${this.options.appName}`,
        ],
        [
          'Navigate by Apple Maps',
          `http://maps.apple.com/?ll=${distLocation.lat},${distLocation.lng}&q=${distLocation.title}&dirflg=${mode === 'drive' ? 'd' : (mode === 'bus' ? 'r' : 'w')}`,
        ],
        [
          'Navigate by Google Maps',
          `comgooglemaps://?saddr=${srcLocation ? (srcLocation.lat + ',' + srcLocation.lng) : ''}&daddr=${distLocation.lat},${distLocation.lng}&directionsmode=${mode === 'drive' ? 'driving' : (mode === 'bus' ? 'transit' : 'walking')}`,
        ]
      ],
    });
  },

  /**
   * Launch the navigation
   *
   * @param destLocation {lat:40, lng: 118, type: 'gcj02', title: '终点'}
   */
  navigate(distLocation) {
    return openDialog({
      android: [
        [
          'Navigate by Gaode Maps',
          `androidamap://navi?sourceApplication=${this.options.appName}&poiname=${distLocation.title}&lat=${distLocation.lat}&lon=${distLocation.lng}&dev=${distLocation.type === 'gcj02' ? '0' : '1'}`,
        ],
        [
          'Navigate by Baidu Maps',
          `bdapp://map/direction?origin=&destination=${distLocation.lat},${distLocation.lng}&mode=driving&coord_type=${distLocation.type === 'gcj02' ? 'gcj02' : 'wgs84'}&src=${this.options.appName}`,
        ]
      ],
      ios: [
        [
          'Navigate by Gaodu Maps',
          `iosamap://navi?sourceApplication=${this.options.appName}&poiname=${distLocation.title}&lat=${distLocation.lat}&lon=${distLocation.lng}&dev=${distLocation.type === 'gcj02' ? '0' : '1'}`,
        ],
        [
          'Navigate by Baidu Maps',
          `baidumap://map/direction?origin=&destination=${distLocation.lat},${distLocation.lng}&mode=driving&coord_type=${distLocation.type === 'gcj02' ? 'gcj02' : 'wgs84'}&src=${this.options.appName}`,
        ],
        [
          'Navigate by Apple Maps',
          `http://maps.apple.com/?ll=${distLocation.lat + ',' + distLocation.lng}&q=${distLocation.title}&dirflg=d`,
        ],
      ],
    });
  },
};
